/* Copyright (C) 2004 - 2005  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
using System;
using com.db4o.config;

namespace com.db4o.test.cs
{
	public class OnActivateEventStrategy
	{
		public event EventHandler Crash;

		public void objectOnActivate(ObjectContainer container)
		{
			Tester.ensureEquals(null, Crash);
			Crash += new EventHandler(Boom);
		}

		public void RaiseCrash()
		{   
			if (null != Crash)
			{
				Crash(this, EventArgs.Empty);
			}
		}

		static string Message = null;

		public static void Prepare()
		{
			Message = null;
		}

		public static void Assert()
		{
			Tester.ensureEquals("Boom!!!!", OnActivateEventStrategy.Message);
		}

		static void Boom(object sender, EventArgs args)
		{
			Message = "Boom!!!!";
		}
	}

	public class CsDelegate
	{
        public event EventHandler Bang;

		static string Message = null;

		public void RaiseBang()
		{
			Bang(this, EventArgs.Empty);
		}

        public void storeOne()
		{
			this.Bang += new EventHandler(OnBang);
        }

        public void testOne()
		{
			// delegate fields are simply not stored
			Tester.ensureEquals(null, Bang);
        }

		public void testTSerializableForDelegate()
		{
			Db4o.configure().objectClass(typeof(EventHandler)).translate(new com.db4o.config.TSerializable());			
			try
			{
				Tester.deleteAllInstances(typeof(CsDelegate));
				Tester.reOpen();

				CsDelegate.Message = null;
				CsDelegate obj = new CsDelegate();
				obj.Bang += new EventHandler(OnBang);
				Tester.store(obj);

				obj = (CsDelegate)Tester.getOne(typeof(CsDelegate));
				obj.RaiseBang();
				Tester.ensureEquals("Bang!!!!", CsDelegate.Message);
			}
			finally
			{
				Db4o.configure().objectClass(typeof(EventHandler)).translate(new TNull());
			}
		}

		/* TODO: See why this is not working
		public void testOnActivateEventStrategy()
		{
			Tester.deleteAllInstances(typeof(OnActivateEventStrategy));
			Tester.store(new OnActivateEventStrategy());
            
			OnActivateEventStrategy.Prepare();
			OnActivateEventStrategy obj = (OnActivateEventStrategy)Tester.objectContainer().get(typeof(OnActivateEventStrategy)).next();
			obj.RaiseCrash();
			OnActivateEventStrategy.Assert();
		}
		*/

		static void OnBang(object sender, EventArgs args) 
		{
			CsDelegate.Message = "Bang!!!!";
		}
	}
}
